begin;

-- add table definition for prd_mta_action
create table prd_mta_action (
    id            serial not null,
    name          varchar(64) not null
);

-- add primary key and constraint for prd_mta_action
alter table prd_mta_action
    add constraint pk_prd_mta_action primary key (id);
alter table prd_mta_action
    add constraint unq_prd_mta_action unique (name);

-- populate prd_mta_action table with constant data
insert into prd_mta_action (name) values ('reject');
insert into prd_mta_action (name) values ('tempfail');
insert into prd_mta_action (name) values ('ok');

-- add table definition for prd_mta_action_count
create table prd_mta_action_count (
    period_id     integer not null,
    pmx_host_id   integer not null,
    relay_id      integer not null,
    mta_action_id integer not null,
    tally         integer
);
    
-- create index on prd_mta_action_count
create index prd_mta_action_count_period_id_pmx_host_id_mta_action_id_index on prd_mta_action_count using btree (period_id, pmx_host_id, mta_action_id);

-- add table definition for prd_policy_action
create table prd_policy_action (
    id            serial not null,
    name          varchar(64) NOT NULL
);

-- add primary key and constraint for prd_policy_action
alter table prd_policy_action
    add constraint pk_prd_policy_action primary key (id);
alter table prd_policy_action
    add constraint unq_prd_policy_action unique (name);

-- populate prd_policy_action table with constant data
insert into prd_policy_action (name) values ('reject');
insert into prd_policy_action (name) values ('discard');
insert into prd_policy_action (name) values ('ok');

-- add table definition for prd_policy_reason
create table prd_policy_reason (
    id            serial not null,
    name          varchar(64) not null
);

-- add primary key and constraint for prd_policy_reason
alter table prd_policy_reason
    add constraint pk_prd_policy_reason primary key (id);
alter table prd_policy_reason
    add constraint unq_prd_policy_reason unique (name);

-- add two columns to prd_msgs_processed_count
alter table prd_msgs_processed_count add column policy_reason_id integer;
alter table prd_msgs_processed_count add column policy_action_id integer;

-- create index for prd_policy_mark_count
create index prd_policy_mark_count_period_id_index on prd_policy_mark_count using btree (period_id);

-- create views used to generate actual reports
create view prd_mta_action_count_view
as
select     prd_period.id as period_id,
           prd_pmx_host.id as pmx_host_id,
           prd_mta_action.id as mta_action_id,
           prd_period.period_start,
           prd_period.period_end,
           prd_period.duration,
           coalesce(GROUPED_ACTION_COUNT.tally,0) as tally
from       prd_period
cross join prd_mta_action
cross join prd_pmx_host
left join  (
            select  period_id,
                    pmx_host_id,
                    mta_action_name,
                    sum(tally) as tally
            from (
                  select period_id,
                         pmx_host_id,
                         case prd_mta_action.name
                              when 'ok' then 'ok' else 'reject'
                         end
                         as mta_action_name,
                         tally
                  from   prd_mta_action_count
                  join   prd_mta_action
                      on prd_mta_action.id = prd_mta_action_count.mta_action_id
                 ) as ACTION_COUNT
            group by period_id, pmx_host_id, mta_action_name
           ) as GROUPED_ACTION_COUNT
    on    GROUPED_ACTION_COUNT.period_id = prd_period.id
     and  GROUPED_ACTION_COUNT.mta_action_name = prd_mta_action.name
     and  GROUPED_ACTION_COUNT.pmx_host_id = prd_pmx_host.id
where     prd_mta_action.name in ('ok','reject')
;

create view prd_policy_action_count_view
as
select     prd_period.id as period_id,
           prd_pmx_host.id as pmx_host_id,
           prd_policy_action.id as policy_action_id,
           prd_period.period_start,
           prd_period.period_end,
           prd_period.duration,
           coalesce(GROUPED_ACTION_COUNT.tally,0) as tally
from       prd_period
cross join prd_policy_action
cross join prd_pmx_host
left join  (
            select  action_count.period_id,
                    action_count.pmx_host_id,
                    action_count.policy_action_name,
                    sum(action_count.tally) as tally
            from (
                  select period_id,
                         pmx_host_id,
                         tally,
                         case when policy_action_id is null
                              then 'ok' else 'reject'
                         end
                         as policy_action_name
                  from   prd_msgs_processed_count
                 ) as ACTION_COUNT
            group by ACTION_COUNT.period_id, ACTION_COUNT.pmx_host_id, ACTION_COUNT.policy_action_name
           ) as GROUPED_ACTION_COUNT
    on     GROUPED_ACTION_COUNT.period_id = prd_period.id
     and   GROUPED_ACTION_COUNT.policy_action_name = prd_policy_action.name
     and   GROUPED_ACTION_COUNT.pmx_host_id = prd_pmx_host.id
where      prd_policy_action.name in ('ok', 'reject')
;

-- change stored procedures definitions
create or replace function del_prd_cfg_data() returns void as '
    delete from prd_applicable_dimension;
    delete from prd_dimension;
    delete from prd_direction;
    delete from prd_fact;
    delete from prd_nullified_dimension;
    delete from prd_reduction_action;
    delete from prd_runtime_fact_type;
    delete from prd_mta_action;
    delete from prd_policy_action;
' language sql;

create or replace function ins_static_prd_cfg_data() returns void as '
    -- prd_user_action.id values chosen to match on-line encodings
    insert into prd_direction
        (name) values (''incoming'');
    insert into prd_direction
        (name) values (''outgoing'');
    insert into prd_dimension
        (name, is_purgeable) values (''recipient'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''sender'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''probability_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''msg_size_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''pmx_host'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''pmx_process'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''spam_virus_rule'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''period'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''fact_type'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''relay'', 1);
    insert into prd_dimension
        (name, is_purgeable) values (''rsrc_utlztn_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''concurrency_range'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''quarantine_reason'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''user_action'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''direction'', 0);
    insert into prd_dimension
        (name, is_purgeable) values (''atchmt_type'', 0);
    insert into prd_dimension
            (name, is_purgeable) values (''mta_action'', 1);
    insert into prd_dimension
            (name, is_purgeable) values (''policy_action'', 1);
    insert into prd_dimension
            (name, is_purgeable) values (''policy_reason'', 1);
    insert into prd_fact
        (name, is_collected) values (''msgs_processed_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''atchmt_processed_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''rule_hit_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''user_action_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''pmx_runtime_fact'', 1);
    insert into prd_fact
        (name, is_collected) values (''policy_mark_count'', 1);
    insert into prd_fact
        (name, is_collected) values (''mta_action_count'', 1);
    insert into prd_runtime_fact_type
        (id, name) values (1, ''disc_kbytes_written'');
    insert into prd_runtime_fact_type
        (id, name) values (2, ''disc_kbytes_read'');
    insert into prd_runtime_fact_type
        (id, name) values (3, ''disc_kbytes_allocated'');
    insert into prd_runtime_fact_type
        (id, name) values (4, ''ram_allocated'');
    insert into prd_runtime_fact_type
        (id, name) values (5, ''swap_allocated'');
    insert into prd_runtime_fact_type
        (id, name) values (6, ''cpu_utilized'');
    insert into prd_runtime_fact_type
        (id, name) values (7, ''load'');
    insert into prd_runtime_fact_type
        (id, name) values (8, ''uptime'');
    insert into prd_runtime_fact_type
        (id, name) values (9, ''total_quarantined_local'');
    insert into prd_runtime_fact_type
        (id, name) values (10, ''total_quarantined_central'');
    insert into prd_runtime_fact_type
        (id, name) values (11, ''new_quarantined_local'');
    insert into prd_runtime_fact_type
        (id, name) values (12, ''new_quarantined_central'');
    insert into prd_runtime_fact_type
        (id, name) values (13, ''quarantine_kbytes_local'');
    insert into prd_runtime_fact_type
        (id, name) values (14, ''quarantine_kbytes_central'');
    insert into prd_runtime_fact_type
        (id, name) values (15, ''disc_kbytes_free'');
    insert into prd_mta_action
        (name) values (''reject'');
    insert into prd_mta_action
        (name) values (''tempfail'');
    insert into prd_mta_action
        (name) values (''ok'');
    insert into prd_policy_action
        (name) values (''reject'');
    insert into prd_policy_action
        (name) values (''discard'');
    insert into prd_policy_action
        (name) values (''ok'');
' language sql;

create or replace function ins_transient_prd_cfg_data() returns void as '
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''sender'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''relay'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''probability_range'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''msg_size_range'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''direction'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''spam_virus_rule'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''quarantine_reason'', ''msgs_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''msg_size_range'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''atchmt_type'', ''atchmt_processed_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''quarantine_reason'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''recipient'', ''msgs_prcsd_per_rcpnt_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''rule_hit_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''rule_hit_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''spam_virus_rule'', ''rule_hit_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''recipient'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''user_action'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''relay'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''sender'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''probability_range'', ''user_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''fact_type'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_process'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''concurrency_range'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''rsrc_utlztn_range'', ''pmx_runtime_fact'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''period'', ''mta_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''relay'', ''mta_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''pmx_host'', ''mta_action_count'', 1);
    insert into prd_applicable_dimension
        (dimension_name, fact_name, is_collected)
            values (''mta_action'', ''mta_action_count'', 1);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''rule_hit_count'', ''rule_hit_count_expiry'',
                    ''expiry'', 172800, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''policy_mark_count'', ''policy_mark_count_expiry'',
                    ''expiry'', 172800, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''pmx_runtime_fact'', ''pmx_runtime_fact_aggregation'',
                    ''aggregation/avg'', 172800, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''pmx_runtime_fact'', ''user_action_count_expiry'',
                    ''expiry'', 63115200, ''daily'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''user_action_count'', ''user_action_count_aggregation'',
                    ''aggregation/sum'', 172800, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''user_action_count'', ''user_action_count_expiry'',
                    ''expiry'', 31557600, ''daily'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
         recurrence, aggregation_period)
            values (''msgs_processed_count'', ''msg_proc_expiry'',
                    ''expiry'', 63115200, ''daily'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''msgs_processed_count'', ''msg_proc_aggregation'',
                    ''aggregation/sum'', 129600, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
         recurrence, aggregation_period)
            values (''msgs_prcsd_per_rcpnt_count'', ''msg_proc_rcpnt_expiry'',
                    ''expiry'', 129600, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
         recurrence, aggregation_period)
            values (''atchmt_processed_count'', ''atchmt_processed_expiry'',
                    ''expiry'', 129600, ''hourly'', null);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''mta_action_count'', ''mta_action_aggregation'',
                    ''aggregation/sum'', 172800, ''hourly'', 3600);
    insert into prd_reduction_action
        (fact_name, action_name, action_type, age_threshold,
        recurrence, aggregation_period)
            values (''mta_action_count'', ''mta_action_expiry'',
                    ''expiry'', 63115200, ''daily'', null);
    insert into prd_nullified_dimension
        (fact_name, action_name, dimension_name)
            values (''msgs_processed_count'', ''msg_proc_aggregation'', ''relay'');
    insert into prd_nullified_dimension
        (fact_name, action_name, dimension_name)
            values (''msgs_processed_count'', ''msg_proc_aggregation'', ''sender'');
' language sql;

-- add new dimensions to prd_dimension
insert into prd_dimension (name,is_purgeable) values ('mta_action',0);
insert into prd_dimension (name,is_purgeable) values ('policy_action',0);
insert into prd_dimension (name,is_purgeable) values ('policy_reason',0);

-- add new facts to prd_fact
insert into prd_fact values ('mta_action_count',1);

-- add reduction actions to prd_reduction_action
insert into prd_reduction_action (fact_name,action_name,action_type,age_threshold,recurrence,aggregation_period) values ('mta_action_count','mta_action_aggregation','aggregation/sum',172800,'hourly',3600);
insert into prd_reduction_action (fact_name,action_name,action_type,age_threshold,recurrence,aggregation_period) values ('mta_action_count','mta_action_expiry','expiry',63115200,'daily',null);

-- update prd_applicable_dimmension for dimensions/fact correlation
insert into prd_applicable_dimension (dimension_name,fact_name,is_collected) values ('period','mta_action_count',1);
insert into prd_applicable_dimension (dimension_name,fact_name,is_collected) values ('pmx_host','mta_action_count',1);
insert into prd_applicable_dimension (dimension_name,fact_name,is_collected) values ('relay','mta_action_count',1);
insert into prd_applicable_dimension (dimension_name,fact_name,is_collected) values ('mta_action','mta_action_count',1);

drop index prd_msgs_processed_count_period_id_index;
create index prd_msgs_processed_count_period_id_pmx_host_id_quarantine_index
    on prd_msgs_processed_count(period_id,pmx_host_id,quarantine_reason_id);

COMMENT ON SCHEMA public IS 'PMX 5.100';

commit;
