begin;

-- Add new report tables
CREATE TABLE prd_state (
    name text NOT NULL,
    asof timestamptz DEFAULT '1970-01-01 00:00:00+00'::timestamptz NOT NULL
);

COPY prd_state (name, asof) FROM stdin;
rep-append	1970-01-01 00:00:00+00
rep-stable	1970-01-01 00:00:00+00
\.

CREATE UNIQUE INDEX prd_state_name_index ON prd_state (name);

create table prd_msgs_by_type (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    "type" text NOT NULL,
    inbound boolean NOT NULL,
    tally integer NOT NULL
);
create index prd_msgs_by_type_index 
    on prd_msgs_by_type (group_name, period_start, type, inbound);

create table prd_msgs_by_virus (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    virus_name text NOT NULL,
    inbound boolean NOT NULL,
    tally integer NOT NULL
);
create index prd_msgs_by_virus_index 
    on prd_msgs_by_virus (group_name, period_start, virus_name, inbound);


create table prd_connections_by_action (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    action text NOT NULL,
    tally integer NOT NULL
);
create index prd_connections_by_action_index 
    on prd_connections_by_action (group_name, period_start, action);

create table prd_relays_by_type (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    "type" text NOT NULL,
    relay text NOT NULL,
    inbound bool NOT NULL,
    tally integer NOT NULL
);
create index prd_relays_by_type_index 
    on prd_relays_by_type (group_name, period_start, type, relay, inbound);

create table prd_msgs_by_policy_block (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    blocked bool NOT NULL,
    tally integer NOT NULL
);
create index prd_msgs_by_policy_block_index 
    on prd_msgs_by_policy_block (group_name, period_start, blocked);

create table prd_spam_by_recipient (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    recipient text NOT NULL,
    tally integer NOT NULL
);
create index prd_spam_by_recipient_index 
    on prd_spam_by_recipient (group_name, period_start, recipient);

create table prd_senders_by_type (
    group_name text NOT NULL,
    period_start timestamptz NOT NULL,
    type text NOT NULL,
    sender text NOT NULL,
    inbound bool NOT NULL,
    tally integer NOT NULL
);
create index prd_senders_by_type_index 
    on prd_senders_by_type (group_name, period_start, type, sender, inbound);

-- New functions for Sophos-Reports based reporting

CREATE OR REPLACE FUNCTION range_interval(timestamptz, timestamptz, integer) RETURNS interval
    AS '
    SELECT
        CASE
            WHEN ($2 - $1)/$3 < INTERVAL ''1 hour'' THEN ''5 min''
            WHEN ($2 - $1)/$3 < INTERVAL ''4 hours'' THEN ''1 hour''
            WHEN ($2 - $1)/$3 < INTERVAL ''12 hours'' THEN ''4 hours''
            WHEN ($2 - $1)/$3 < INTERVAL ''1 day'' THEN ''12 hours''
            WHEN ($2 - $1)/$3 < INTERVAL ''2 days'' THEN ''1 day''
            WHEN ($2 - $1)/$3 < INTERVAL ''7 days'' THEN ''2 days''
            WHEN ($2 - $1)/$3 < INTERVAL ''2 weeks'' THEN ''1 week''
            WHEN ($2 - $1)/$3 < INTERVAL ''4 weeks'' THEN ''2 weeks''
            WHEN ($2 - $1)/$3 < INTERVAL ''8 weeks'' THEN ''4 weeks''
            ELSE ''8 weeks''
        END::interval;
'   
    LANGUAGE sql IMMUTABLE STRICT;


CREATE OR REPLACE FUNCTION floor_period(timestamptz, timestamptz, timestamptz, integer) RETURNS timestamptz
    AS '
    SELECT $2 + floor(extract(epoch from $1-$2)/secs)::int * secs * interval''1 sec''
    FROM (SELECT extract(epoch FROM range_interval($2,$3,$4)) AS secs) AS x;
'   
    LANGUAGE sql IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION generate_series(int,int,int) RETURNS SETOF int
   AS '
   DECLARE start ALIAS FOR $1;
           stop  ALIAS FOR $2;
           hop   ALIAS FOR $3;
           i     int;
   BEGIN
        i := start;
        WHILE i <= stop LOOP
                RETURN NEXT i;
                i := i + hop;
        END LOOP;
        RETURN;
   END
'
        LANGUAGE plpgsql IMMUTABLE;

CREATE OR REPLACE FUNCTION period_seq(timestamptz, timestamptz, interval) RETURNS SETOF timestamptz
    AS '
    SELECT  $1 + secs*INTERVAL''1 sec'' AS period
    FROM    generate_series(0, EXTRACT(EPOCH FROM $2-$1)::int,
                            EXTRACT(EPOCH FROM $3)::int) AS seq(secs);
'   
    LANGUAGE sql IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION period_seq_grain(timestamptz, timestamptz, integer) RETURNS SETOF timestamptz
    AS '
    SELECT * FROM period_seq($1, $2, range_interval($1, $2, $3));
'   
    LANGUAGE sql IMMUTABLE STRICT;

CREATE TYPE val_freq_t AS (
	val text,
	freq real
);

CREATE FUNCTION val_freq_set(text, text) RETURNS SETOF val_freq_t
    AS '
DECLARE tbl ALIAS FOR $1;
        col ALIAS FOR $2;
        vals        text[];
        freqs       real[];
        result      val_freq_t;
        i           int;
BEGIN
    SELECT  most_common_vals, most_common_freqs
    INTO    vals, freqs
    FROM    pg_stats
    WHERE   tablename = tbl AND attname = col;

    IF FOUND THEN
        FOR i IN array_lower(vals,1)..array_upper(vals,1) LOOP
            result.val  := vals[i];
            result.freq := freqs[i];
            RETURN NEXT result;
        END LOOP;
    END IF;
 
    RETURN;
END
'
    LANGUAGE plpgsql;
COMMENT ON FUNCTION val_freq_set(text, text) IS 'Returns most-common (val,freq) statistics from pg_stats, for a given table+column';

-- Group Policy Option tables

create table group_policy_option_opt_in (
    name text NOT NULL,
    description text NOT NULL,
    default_on boolean NOT NULL,
    opt_in boolean NOT NULL,
    list_name text NOT NULL
);

create unique index group_policy_option_opt_in_index ON group_policy_option_opt_in (name);

-- Permissions for new reports

insert into binary_permission_item (permission_item, default_permission_bits) values ('reports.queuestatus', 0);
insert into binary_permission_item (permission_item, default_permission_bits) values ('reports.queuestatus.export', 0);

-- Cascaded delete of resource data

ALTER TABLE cfg_kv ADD CONSTRAINT fk_cfg_kv_cfg_instance 
    FOREIGN KEY (instance_id) REFERENCES cfg_instance ON DELETE CASCADE;
ALTER TABLE cfg_listrow ADD CONSTRAINT fk_cfg_listrow_cfg_instance 
    FOREIGN KEY (instance_id) REFERENCES cfg_instance ON DELETE CASCADE;
ALTER TABLE cfg_binval ADD CONSTRAINT fk_cfg_binval_cfg_instance 
    FOREIGN KEY (instance_id) REFERENCES cfg_instance ON DELETE CASCADE;
ALTER TABLE cfg_item_change ADD CONSTRAINT fk_cfg_item_change_cfg_instance 
    FOREIGN KEY (instance_id) REFERENCES cfg_instance ON DELETE CASCADE;

-- Tables for message forwarding

create table pmx_forward_config (
    access_right text NOT NULL,
    address text NOT NULL,
    subject text NOT NULL,
    description text NOT NULL
);

insert into pmx_forward_config (access_right, address, subject, description)
  values
    ('quarantine.actions.forward.false_positive', 
      'not-spam@puremessage.net',
     'PM-UNIX false positive report', 
     'Report false positive(s) to SophosLabs');
insert into binary_permission_item (permission_item) values ('quarantine.actions.forward.false_positive');

insert into pmx_forward_config (access_right, address, subject, description)
  values
    ('quarantine.actions.forward.false_negative', 
      'is-spam@puremessage.net',
     'PM-UNIX false negative report', 
     'Report false negative(s) to SophosLabs');
insert into binary_permission_item (permission_item) values ('quarantine.actions.forward.false_negative');

insert into pmx_forward_config (access_right, address, subject, description)
  values
    ('quarantine.actions.forward.potential_virus', 
      'samples@sophos.com',
     'PM-UNIX potential virus report', 
     'Report potential virus(es) to SophosLabs');
insert into binary_permission_item (permission_item) values ('quarantine.actions.forward.potential_virus');

insert into pmx_forward_config (access_right, address, subject, description)
  values
    ('quarantine.actions.forward.support_submission', 
      'puremessage-support@sophos.com',
     'PM-UNIX sample submission to support',
     'Report to Sophos Support');
insert into binary_permission_item (permission_item, default_permission_bits) values ('quarantine.actions.forward.support_submission', 0);

insert into binary_permission_item (permission_item) values ('quarantine.actions.forward.custom');

-- Update permissions

insert into binary_permission_item (permission_item) values ('change_password');
delete from read_write_permission_item where permission_item = 'configuration.eu_lists.blacklisted-senders-per-user';
delete from read_write_permission_item where permission_item = 'configuration.eu_lists.whitelisted-senders-per-user';
delete from binary_permission_item where permission_item = 'configuration.policydescription';
delete from binary_permission_item where permission_item = 'quarantine.actions.report';
delete from binary_permission_item where permission_item = 'reports.numberofreleases';
delete from binary_permission_item where permission_item = 'reports.numberofreleases.export';
delete from binary_permission_item where permission_item = 'reports.policymarkhits';
delete from binary_permission_item where permission_item = 'reports.policymarkhits.export';
delete from binary_permission_item where permission_item = 'reports.rejectedmtaconnections';
delete from binary_permission_item where permission_item = 'reports.rejectedmtaconnections.export';
delete from binary_permission_item where permission_item = 'reports.spamrangevolumes';
delete from binary_permission_item where permission_item = 'reports.spamrangevolumes.export';
delete from binary_permission_item where permission_item = 'reports.spamviruscount';
delete from binary_permission_item where permission_item = 'reports.spamviruscount.export';
delete from binary_permission_item where permission_item = 'reports.topreleasers';
delete from binary_permission_item where permission_item = 'reports.topreleasers.export';

-- Multifile tables

create table multifile (
    multifile_id   text PRIMARY KEY,
    charset        text NOT NULL DEFAULT 'latin1',
    name           text NOT NULL,
    "type"         text NOT NULL
);

create table group_multifile (
    group_multifile_id serial PRIMARY KEY,
    multifile_id     text NOT NULL REFERENCES multifile (multifile_id) ON DELETE CASCADE ON UPDATE CASCADE,
    group_name       text NOT NULL REFERENCES pmx_group (group_name) ON DELETE CASCADE ON UPDATE CASCADE,
    content          text NOT NULL DEFAULT '',
    checksum         text NOT NULL DEFAULT '',
    UNIQUE (multifile_id, group_name)
);

-- For groups global admin

ALTER TABLE pmx_user ADD COLUMN super_user boolean;
ALTER TABLE pmx_user ALTER COLUMN super_user SET DEFAULT false;

-- Create super user group 
insert into pmx_group values ('supergroup', 'Super user group');

-- Store message reason directly in message table
ALTER TABLE message ADD COLUMN reason text;

-- Update/add report permissions
update binary_permission_item
    set permission_item = 'reports.messagecategories'
    where permission_item = 'reports.messagecategorizations';
update binary_permission_item
    set permission_item = 'reports.messagesblocked.export'
    where permission_item = 'reports.ipblockedmessages.export';

update binary_permission_item
    set permission_item = 'reports.messagecategories.export'
    where permission_item = 'reports.messagecategorizations.export';
update binary_permission_item 
    set permission_item = 'reports.messagesblocked'
    where permission_item = 'reports.ipblockedmessages';

insert into binary_permission_item (permission_item) 
    values ('reports.topvirussenders');
insert into binary_permission_item (permission_item)
    values ('reports.topvirussenders.export');
insert into binary_permission_item (permission_item) 
    values ('reports.categoriesbygroup');
insert into binary_permission_item (permission_item) 
    values ('reports.categoriesbygroup.export');


COMMENT ON SCHEMA public IS 'PMX 5.302';
commit;
